from NanoTCAD_ViDES import *
rank = MPI.COMM_WORLD.Get_rank()

# I create the Hamiltonian for the GNR
a=loadtxt("Wannier2ViDES.H")
h2=array(a,dtype="complex")
n=28;
Nc=32;
GNR=Hamiltonian(n,Nc)
GNR.H=h2
# I translate the bands by the value 3.42809
GNR.H[1:n*Nc+1,2]=GNR.H[1:n*Nc+1,2]+3.42809


GNR.dE=1e-3;
GNR.Phi=zeros(n*Nc);
GNR.Ei=zeros(n*Nc);

# I read the atoms coordinates
b=loadtxt("Wannier2ViDES.xyz")
GNR.x=around(b[:,0]/10.0,2)
GNR.y=around(b[:,1]/10.0,2)
GNR.z=around(b[:,2]/10.0,2)
del b

# I create the grid
x=nonuniformgrid(array([-1,0.4,0,0.1,1,0.4]));
y1=nonuniformgrid(array([-1,0.4,0,0.1]));
y2=nonuniformgrid(array([max(GNR.y),0.1,max(GNR.y)+1,0.4]));
y=concatenate((y1,y2));
grid=grid3D(x,y,GNR.z,GNR.x,GNR.y,GNR.z);

# Now I define the gate regions
top_gate=gate("hex",grid.xmax,grid.xmax,grid.ymin,grid.ymax,2,12)
bottom_gate=gate("hex",grid.xmin,grid.xmin,grid.ymin,grid.ymax,2,12)
source_gate=gate("hex",-0.1,0.1,min(GNR.y),max(GNR.y),-1,2)
drain_gate=gate("hex",-0.1,0.1,min(GNR.y),max(GNR.y),12,15)

GNR.mu1=0;
GNR.mu2=-0.3;

offset=0.5;
top_gate.Ef=0;
bottom_gate.Ef=0;
source_gate.Ef=-offset;
drain_gate.Ef=-offset+GNR.mu2;

# I take care of the solid
SiO2=region("hex",grid.xmin,grid.xmax,grid.ymin,grid.ymax,grid.zmin,grid.zmax);
SiO2.eps=3.9;

# I create the interface
#p=interface3D(grid,top_gate,lateral_gate1,lateral_gate2,SiO2);
p=interface3D(grid,top_gate,bottom_gate,source_gate,drain_gate,SiO2);

#I find the initial solution
solve_Poisson(grid,p);
section("x",p.Phi,0,grid);
savetxt("Phi.ini",p.Phi);

# I start the Vgs sweep with 
# 0.05V as voltage step
Vgsmin=0.0;
Vgsmax=1.0;
Vgstep=0.05;

#I create the vectors in which I store the data
vd=zeros(20);
current=zeros(20);


GNR.Elower=-0.5;
GNR.Eupper=0.5;

p.MPI="yes"

counter=0;
Vgs=Vgsmin;
while (Vgs<=Vgsmax):

    # I set the Fermi level of the top and bottom gate
    top_gate.Ef=-Vgs;
    set_gate(p,top_gate);
    bottom_gate.Ef=-Vgs;
    set_gate(p,bottom_gate);

    p.normpoisson=1e-1;
    p.normd=3e-2;
    solve_self_consistent(grid,p,GNR);
    vd[counter]=Vgs;
    #computing the current
    vt=kboltz*300.0/q;
    E=array(GNR.E);
    T=array(GNR.T);
    arg=2*q*q/(2*pi*hbar)*T*(Fermi((E-GNR.mu1)/vt)-Fermi((E-GNR.mu2)/vt))*GNR.dE
    id=sum(arg);
    current[counter]=id;
    if (rank==0):
        string="./datiout/Phi%s.out" %Vgs;
        savetxt(string,p.Phi);
        string="./datiout/ncar%s.out" %Vgs;
        savetxt(string,p.free_charge);
        del a;
        a=[GNR.E,GNR.T];
        string="./datiout/T%s.out" %Vgs;
        savetxt(string,transpose(a));
        del a
        a=[GNR.z,-p.Phi[grid.swap]];
        string="./datiout/Ei%s.out" %Vgs;
        savetxt(string,transpose(a));
        string="./datiout/jayn%s.out" %Vgs;
        fp=open(string,"w");
        string2="%s" %current[counter];
        fp.write(string2);
        fp.close();


    counter=counter+1;
    Vgs=Vgs+Vgstep;

tempo=[vd,current]
savetxt("./datiout/idvds.out",transpose(tempo));

